#include "textfile.h"
#include "jtypes.h"

class HSIPalFile {

public:

    HSIPalFile();
   ~HSIPalFile();
    u32 load(i8 *filename);
    u32 save(i8 *filename);
    u32 paletteLen();
    u32 copyPalette18bit (
        u8 *dst,
        u32 a,
        u32 len
        );
    u32 copyPalette24bit (
        u8 *dst,
        u32 a,
        u32 len
        );
    u32 setColor (
        u8 color,
        u8 r,
        u8 g,
        u8 b
        );

private:

    u32 checkSyntax();

    u32         valid;
    u32         palLEN;
    TextFile    colorDEF;
    u8          pal[768];

};

/*
    [function]  Check syntax and construct palette along the way.
    [returns]   0:success
`.....................................................................*/
u32 HSIPalFile::checkSyntax()
{
    i32         i0,i1;
    u32         u0,u1;
    u32         rr,gg,bb;
    i8          line[80];

    colorDEF.moveCursorABS(1);
    colorDEF.currentLine(line,80);
    if (strcmp(line,"PAL") != 0) return 1;

    colorDEF.moveCursorABS(2);
    colorDEF.currentLine(line,80);
    i0 = sscanf(line,"%d",&i1);
    if (i0 == 0) return 2;

    if ((i1 > 256) || (i1 < 2)) return 3;

    if ((u32) (i1 + 3) < colorDEF.lastLineNUM()) return 4;

    u1 = 0;
    for (u0 = 3; u0 < (u32) (i1 + 3); u0++) {
        colorDEF.moveCursorABS(u0);
        colorDEF.currentLine(line,80);
        i0 = sscanf(line,"%d %d %d",&rr,&gg,&bb);
        if (i0 != 3) return 5;
        pal[u1+0] = rr;
        pal[u1+1] = gg;
        pal[u1+2] = bb;
        u1 += 3;
    }

    palLEN = i1;
    valid = 1;
    return 0;
}


// API :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


/*
    [function]
`.....................................................................*/
HSIPalFile::HSIPalFile()
{
    valid = 0;
    palLEN = 0;
    memset(pal,0xff,768);
}


/*
    [function]
`.....................................................................*/
HSIPalFile::~HSIPalFile()
{

}


/*
    [function]  loads an HSI Palette file.
    [returns]   0:success, 1:fileNoExiste, 2:syntaxError,
                3:alreadyLoaded
`.....................................................................*/
u32 HSIPalFile::load(i8 *filename)
{
    u32 failure;

    if (this->valid) return 3;
    failure = colorDEF.load(filename,Rx_ACCESS);
    if (failure) {
        valid = 0;
        palLEN = 0;
        return 1;
    }
    failure = checkSyntax();
    if (failure) return 2;
    return 0;
}


/*
    [function]  loads an HSI Palette file.
    [returns]   0:success, 1:error
`.....................................................................*/
u32 HSIPalFile::save(i8 *filename)
{
    u32         u0,u1,u2;
    char        str[32];
    TextFile    f0;

    u0 = 0;
    u0 = f0.phresh(filename,(palLEN+2),xW_ACCESS);
    if (u0) return u0;
    f0.moveCursorABS(1);
    f0.overwriteLine("PAL");
    sprintf(str,"%d",palLEN);
    f0.moveCursorREL(1);
    f0.overwriteLine(str);
    u1 = palLEN + 3;
    if (palLEN) {
        for (u0 = 3; u0 < u1; u0++) {
            u2 = (u0 - 3) * 3;
            f0.moveCursorABS(u0);
            sprintf(str,"%4d%4d%4d",pal[u2],pal[u2+1],pal[u2+2]);
            f0.overwriteLine(str);
        }
    }
    f0.save();
    return 0;
}


/*
    [function]
`.....................................................................*/
u32 HSIPalFile::paletteLen()
{
    return palLEN;
}


/*
    [function]
    [returns]   0:success, 1:badRange
`.....................................................................*/
u32 HSIPalFile::copyPalette18bit (
    u8 *dst,
    u32 a,
    u32 len
    )
{
    u32 u0,u1,u2,u3,u4,u5;

    if ((len - a) > palLEN) return 1;

    u1 = a * 3;
    u2 = 0;

    for (u0 = 0; u0 < len; u0++) {
        u3 = pal[u1+0];
        u4 = pal[u1+1];
        u3 >>= 2;
        u5 = pal[u1+2];
        u4 >>= 2;
        u1 += 3;
        u5 >>= 2;
        dst[u2+0] = u3;
        dst[u2+1] = u4;
        dst[u2+2] = u5;
        u2 += 3;
    }
    return 0;
}


/*
    [function]
    [returns]   0:success, 1:badRange
`.....................................................................*/
u32 HSIPalFile::copyPalette24bit (
    u8 *dst,
    u32 a,
    u32 len
    )
{
    if ((len - a) > palLEN) return 1;
    memcpy(dst,&pal[a*3],(len * 3));
    return 0;
}


/*
    [function]  Sets palette entries.
    [returns]   0:success
`.....................................................................*/
u32 HSIPalFile::setColor (
    u8 color,
    u8 r,
    u8 g,
    u8 b
    )
{
    u32 len = (u32) color + 1;

    palLEN = (len > palLEN) ? len : palLEN;
    pal[color * 3 + 0] = r;
    pal[color * 3 + 1] = g;
    pal[color * 3 + 2] = b;
    return 0;
}

